TERM_PREGNANCY_LENGTH_DAYS = 40 * 7
TERM_LOWER_THRESHOLD_LENGTH_DAYS = 37 * 7
EXTREME_PREMATURITY_THRESHOLD_LENGTH_DAYS = 32 * 7
TWENTY_TWO_WEEKS_GESTATION = ((22 * 7) - (40 * 7)) / 365.25  # 22 weeks as decimal age
TWENTY_FIVE_WEEKS_GESTATION = ((25 * 7) - (40 * 7)) / 365.25  # 25 weeks as decimal age
THIRTY_SEVEN_WEEKS_GESTATION = ((37 * 7) - (40 * 7)) / 365.25  # 37 weeks as decimal age
FORTY_TWO_WEEKS_GESTATION = ((42 * 7) - (40 * 7)) / 365.25  # 42 weeks as decimal age
TWENTY_THREE_WEEKS_GESTATION = ((23 * 7) - (40 * 7)) / 365.25  # 23 weeks as decimal age
FIFTY_WEEKS_GESTATION = ((50 * 7) - (40 * 7)) / 365.25  # 50 weeks as decimal age
ZERO_YEARS = 0.0
TWO_YEARS = 2.0
THREE_YEARS = 3.0
FIVE_YEARS = 5.0
TEN_YEARS = 10.0
SEVENTEEN_YEARS = 17.0
EIGHTEEN_YEARS = 18.0
NINETEEN_YEARS = 19.0
TWENTY_YEARS = 20.0
TERM_WEEKS = 40
LOWER_THRESHOLD_PRETERM_REFERENCE_WEEKS = 23
